#include "price_notify.hpp"
#include <esp_timer.h>

using std::make_unique;

namespace PriceNotify {

PriceNotifyManager::PriceNotifyManager() : currentSource(PriceSource::NONE) {}

PriceNotifyManager::~PriceNotifyManager() {
    disconnect();
}

void PriceNotifyManager::init(PriceSource source) {
    currentSource = source;
    setPriceSource(source);
}

void PriceNotifyManager::connect() {
    if (priceSource) {
        priceSource->connect();
    }
}

void PriceNotifyManager::disconnect() {
    if (priceSource) {
        priceSource->disconnect();
    }
}

bool PriceNotifyManager::isConnected() const {
    return priceSource ? priceSource->isConnected() : false;
}

uint64_t PriceNotifyManager::getPrice(Currency currency) const {
    if (priceSource) {
        return priceSource->getPrice(currency);
    }
    // If no price source, return from internal storage
    auto it = prices.find(currency);
    return (it != prices.end()) ? it->second : 0;
}

uint32_t PriceNotifyManager::getLastUpdate(Currency currency) const {
    if (priceSource) {
        return priceSource->getLastUpdate(currency);
    }
    // If no price source, return from internal storage
    auto it = lastUpdates.find(currency);
    return (it != lastUpdates.end()) ? it->second : 0;
}

void PriceNotifyManager::onPriceUpdate(function<void(Currency, uint64_t)> callback) {
    userCallback = callback;
    if (priceSource) {
        priceSource->onPriceUpdate([this](Currency currency, uint64_t price) {
            if (userCallback) {
                userCallback(currency, price);
            }
        });
    }
}

void PriceNotifyManager::loop() {
    if (priceSource) {
        priceSource->loop();
    }
}

void PriceNotifyManager::setPriceSource(PriceSource source) {
    // Store the callback before destroying the old source
    auto oldCallback = userCallback;
    
    // Disconnect and destroy old source
    if (priceSource) {
        priceSource->disconnect();
    }
    
    // Create new source
    switch (source) {
        case PriceSource::COINCAP:
            priceSource = make_unique<CoinCapSource>();
            break;
        case PriceSource::KRAKEN:
            priceSource = make_unique<KrakenSource>();
            break;
        case PriceSource::NONE:
            priceSource.reset();
            break;
    }
    
    currentSource = source;
    
    // Restore callback if it exists
    if (oldCallback) {
        onPriceUpdate(oldCallback);
    }
}

PriceSource PriceNotifyManager::getCurrentSource() const {
    return currentSource;
}

void PriceNotifyManager::processNewPrice(uint64_t price, Currency currency) {
    // Store the price and timestamp
    prices[currency] = price;
    lastUpdates[currency] = esp_timer_get_time() / 1000000;  // Current time in seconds
    
    // If we have a callback, notify about the price update
    if (userCallback) {
        userCallback(currency, price);
    }
}

} // namespace PriceNotify 