#pragma once

#include "interfaces/price_source.hpp"
#include "sources/coincap_source.hpp"
#include "sources/kraken_source.hpp"
#include <memory>
#include <map>

namespace PriceNotify {

using std::unique_ptr;
using std::function;
using std::map;

enum class PriceSource {
    NONE,  // Added for when no explicit source is set
    COINCAP,
    KRAKEN
};

class PriceNotifyManager {
public:
    PriceNotifyManager();
    ~PriceNotifyManager();

    // Initialize with a specific price source
    void init(PriceSource source);
    
    // Connect to the price source
    void connect();
    
    // Disconnect from the price source
    void disconnect();
    
    // Check if connected to price source
    bool isConnected() const;
    
    // Get the last known price for a currency
    uint64_t getPrice(Currency currency) const;
    
    // Get the last update timestamp for a currency
    uint32_t getLastUpdate(Currency currency) const;
    
    // Set callback for price updates
    void onPriceUpdate(function<void(Currency, uint64_t)> callback);
    
    // Process websocket loop - should be called regularly
    void loop();
    
    // Change the price source
    void setPriceSource(PriceSource source);
    
    // Get current price source
    PriceSource getCurrentSource() const;

    // Process new price from external source
    void processNewPrice(uint64_t price, Currency currency);

private:
    unique_ptr<IPriceSource> priceSource;
    PriceSource currentSource;
    function<void(Currency, uint64_t)> userCallback;
    
    // For storing prices when no explicit source is set
    map<Currency, uint64_t> prices;
    map<Currency, uint32_t> lastUpdates;
};
} // namespace PriceNotify 