#include "coincap_source.hpp"

namespace PriceNotify {

CoinCapSource* CoinCapSource::instance = nullptr;

CoinCapSource::CoinCapSource() : connected(false) {
    instance = this;
}

CoinCapSource::~CoinCapSource() {
    disconnect();
    instance = nullptr;
}

void CoinCapSource::connect() {
    webSocket.beginSSL("ws.coincap.io", 443, "/prices?assets=bitcoin");
    webSocket.onEvent([](WStype_t type, uint8_t* payload, size_t length) {
        if (instance) {
            instance->handleWebSocketEvent(type, payload, length);
        }
    });
    webSocket.setReconnectInterval(5000);
    webSocket.enableHeartbeat(15000, 3000, 2);
}

void CoinCapSource::disconnect() {
    webSocket.disconnect();
    connected = false;
}

bool CoinCapSource::isConnected() const {
    return connected && webSocket.isConnected();
}

uint64_t CoinCapSource::getPrice(Currency currency) const {
    auto it = prices.find(currency);
    return (it != prices.end()) ? it->second : 0;
}

uint32_t CoinCapSource::getLastUpdate(Currency currency) const {
    auto it = lastUpdates.find(currency);
    return (it != lastUpdates.end()) ? it->second : 0;
}

void CoinCapSource::onPriceUpdate(function<void(Currency, uint64_t)> callback) {
    priceUpdateCallback = callback;
}

void CoinCapSource::loop() {
    webSocket.loop();
}

void CoinCapSource::handleWebSocketEvent(WStype_t type, uint8_t* payload, size_t length) {
    switch(type) {
        case WStype_DISCONNECTED:
            Serial.println(F("CoinCap WS Disconnected"));
            connected = false;
            break;
            
        case WStype_CONNECTED:
            Serial.println(F("CoinCap WS Connected"));
            connected = true;
            break;
            
        case WStype_TEXT:
            processMessage((char*)payload);
            break;
            
        default:
            break;
    }
}

void CoinCapSource::processMessage(const char* payload) {
    StaticJsonDocument<512> doc;
    DeserializationError error = deserializeJson(doc, payload);
    
    if (error) {
        Serial.println(F("Failed to parse CoinCap message"));
        return;
    }
    
    if (doc["bitcoin"].is<JsonObject>()) {
        uint64_t price = doc["bitcoin"].as<uint64_t>();
        uint32_t timestamp = esp_timer_get_time() / 1000000;  // Current time in seconds
        
        prices[Currency::USD] = price;
        lastUpdates[Currency::USD] = timestamp;
        
        if (priceUpdateCallback) {
            priceUpdateCallback(Currency::USD, price);
        }
    }
}
} // namespace PriceNotify 