#include "kraken_source.hpp"

namespace PriceNotify {

KrakenSource* KrakenSource::instance = nullptr;

KrakenSource::KrakenSource() : connected(false) {
    instance = this;
}

KrakenSource::~KrakenSource() {
    disconnect();
    instance = nullptr;
}

void KrakenSource::connect() {
    webSocket.beginSSL("ws.kraken.com", 443, "/ws");
    webSocket.onEvent([](WStype_t type, uint8_t* payload, size_t length) {
        if (instance) {
            instance->handleWebSocketEvent(type, payload, length);
        }
    });
    webSocket.setReconnectInterval(5000);
    webSocket.enableHeartbeat(15000, 3000, 2);
}

void KrakenSource::disconnect() {
    webSocket.disconnect();
    connected = false;
}

bool KrakenSource::isConnected() const {
    return connected && webSocket.isConnected();
}

uint64_t KrakenSource::getPrice(Currency currency) const {
    auto it = prices.find(currency);
    return (it != prices.end()) ? it->second : 0;
}

uint32_t KrakenSource::getLastUpdate(Currency currency) const {
    auto it = lastUpdates.find(currency);
    return (it != lastUpdates.end()) ? it->second : 0;
}

void KrakenSource::onPriceUpdate(function<void(Currency, uint64_t)> callback) {
    priceUpdateCallback = callback;
}

void KrakenSource::loop() {
    webSocket.loop();
}

void KrakenSource::handleWebSocketEvent(WStype_t type, uint8_t* payload, size_t length) {
    switch(type) {
        case WStype_DISCONNECTED:
            Serial.println(F("Kraken WS Disconnected"));
            connected = false;
            break;
            
        case WStype_CONNECTED:
            Serial.println(F("Kraken WS Connected"));
            connected = true;
            subscribe();
            break;
            
        case WStype_TEXT:
            processMessage((char*)payload);
            break;
            
        default:
            break;
    }
}

void KrakenSource::subscribe() {
    // Subscribe to XBT/USD ticker
    StaticJsonDocument<256> doc;
    doc["event"] = "subscribe";
    JsonArray pair = doc.createNestedArray("pair");
    pair.add("XBT/USD");
    doc["subscription"]["name"] = "ticker";

    String message;
    serializeJson(doc, message);
    webSocket.sendTXT(message);
}

void KrakenSource::processMessage(const char* payload) {
    StaticJsonDocument<512> doc;
    DeserializationError error = deserializeJson(doc, payload);
    
    if (error) {
        Serial.println(F("Failed to parse Kraken message"));
        return;
    }
    
    // Check if it's a ticker update (array format)
    if (doc.is<JsonArray>() && doc.size() >= 4) {
        JsonArray arr = doc.as<JsonArray>();
        if (arr[2] == "ticker" && arr[3] == "XBT/USD") {
            JsonObject tickerData = arr[1].as<JsonObject>();
            if (tickerData.containsKey("c")) {
                // Get the first value from the "c" array which is the last trade price
                uint64_t price = tickerData["c"][0].as<float>() * 100;  // Convert to cents
                uint32_t timestamp = esp_timer_get_time() / 1000000;  // Current time in seconds
                
                prices[Currency::USD] = price;
                lastUpdates[Currency::USD] = timestamp;
                
                if (priceUpdateCallback) {
                    priceUpdateCallback(Currency::USD, price);
                }
            }
        }
    }
}
} // namespace PriceNotify 